@@ -1,44 +1,11 @@
-# ***** BEGIN LICENSE BLOCK *****
-# Version: MPL 1.1/GPL 2.0/LGPL 2.1
-#
-# The contents of this file are subject to the Mozilla Public License Version
-# 1.1 (the "License"); you may not use this file except in compliance with
-# the License. You may obtain a copy of the License at
-# http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS IS" basis,
-# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
-# for the specific language governing rights and limitations under the
-# License.
-#
-# The Original Code is Mozilla Corporation Code.
-#
-# The Initial Developer of the Original Code is
-# Mikeal Rogers.
-# Portions created by the Initial Developer are Copyright (C) 2008
-# the Initial Developer. All Rights Reserved.
-#
-# Contributor(s):
-#  Mikeal Rogers <mikeal.rogers@gmail.com>
-#
-# Alternatively, the contents of this file may be used under the terms of
-# either the GNU General Public License Version 2 or later (the "GPL"), or
-# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
-# in which case the provisions of the GPL or the LGPL are applicable instead
-# of those above. If you wish to allow use of your version of this file only
-# under the terms of either the GPL or the LGPL, and not to allow others to
-# use your version of this file under the terms of the MPL, indicate your
-# decision by deleting the provisions above and replace them with the notice
-# and other provisions required by the GPL or the LGPL. If you do not delete
-# the provisions above, a recipient may use your version of this file under
-# the terms of any one of the MPL, the GPL or the LGPL.
-#
-# ***** END LICENSE BLOCK *****
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, you can obtain one at http://mozilla.org/MPL/2.0/.
 
 import asyncore
+import inspect
 import socket
 import select
-import logging
 import uuid
 from time import sleep
 from threading import Thread
@@ -50,9 +17,12 @@ except ImportError:
     import simplejson
     from simplejson.encoder import encode_basestring_ascii, encode_basestring
 
-logger = logging.getLogger(__name__)
+import jsobjects
+
+
+class JavaScriptException(Exception):
+    pass
 
-class JavaScriptException(Exception): pass
 
 class Telnet(asyncore.dispatcher):
     def __init__(self, host, port):
@@ -61,7 +31,6 @@ class Telnet(asyncore.dispatcher):
         self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
         self.connect((host, port))
         self.buffer = ''
-        self.logger = logger
 
     def __del__(self):
         self.close()
@@ -70,18 +39,18 @@ class Telnet(asyncore.dispatcher):
         """override method of asyncore.dispatcher"""
         self.close()
 
-    def handle_expt(self): 
-        self.close() # connection failed, shutdown
-    
+    def handle_expt(self):
+        # connection failed, shutdown
+        self.close()
+
     def writable(self):
         return (len(self.buffer) > 0)
 
     def handle_write(self):
         sent = self.send(self.buffer)
         self.buffer = self.buffer[sent:]
-        
+
     def read_all(self):
-        import socket
         data = ''
         while self.connected:
             try:
@@ -92,26 +61,22 @@ class Telnet(asyncore.dispatcher):
         return data
 
     def handle_read(self):
-        self.data = self.read_all()
-        self.process_read(self.data)
-
-        
-    read_callback = lambda self, data: None
+        data = self.read_all()
+        self.process_read(data)
 
 decoder = simplejson.JSONDecoder()
 
+
 class JSObjectEncoder(simplejson.JSONEncoder):
     """Encoder that supports jsobject references by name."""
-
     def encode(self, o):
-        import jsobjects
         if isinstance(o, jsobjects.JSObject):
             return o._name_
         else:
             return simplejson.JSONEncoder.encode(self, o)
 
     def _iterencode(self, o, markers=None):
-        import jsobjects
+        # XXX verbosely copied from simplejson
         if isinstance(o, jsobjects.JSObject):
             yield o._name_
         elif isinstance(o, basestring):
@@ -121,7 +86,7 @@ class JSObjectEncoder(simplejson.JSONEncoder):
                 encoder = encode_basestring
             _encoding = self.encoding
             if (_encoding is not None and isinstance(o, str)
-                    and not (_encoding == 'utf-8')):
+                and not (_encoding == 'utf-8')):
                 o = o.decode(_encoding)
             yield encoder(o)
         elif o is None:
@@ -133,7 +98,8 @@ class JSObjectEncoder(simplejson.JSONEncoder):
         elif isinstance(o, (int, long)):
             yield str(o)
         elif isinstance(o, float):
-            yield getattr(simplejson.encoder, 'floatstr', simplejson.encoder._floatstr)(o, self.allow_nan)
+            yield getattr(simplejson.encoder, 'floatstr',
+                          simplejson.encoder._floatstr)(o, self.allow_nan)
         elif isinstance(o, (list, tuple)):
             for chunk in self._iterencode_list(o, markers):
                 yield chunk
@@ -153,24 +119,26 @@ class JSObjectEncoder(simplejson.JSONEncoder):
 
 encoder = JSObjectEncoder()
 
-class JSBridgeDisconnectError(Exception): 
+
+class JSBridgeDisconnectError(Exception):
     """exception raised when an unexpected disconect happens"""
 
 
 class Bridge(Telnet):
-    
     trashes = []
     reading = False
     sbuffer = ''
     events_list = []
 
     callbacks = {}
-        
+
     bridge_type = "bridge"
-    
+
     registered = False
-    timeout_ctr = 0. # global timeout counter
-    
+
+    # global timeout counter
+    timeout_ctr = 0.
+
     def __init__(self, host, port, timeout=60.):
         """
         - timeout : failsafe timeout for each call to run in seconds
@@ -181,7 +149,7 @@ class Bridge(Telnet):
 
         # XXX we've actually already connected in Telnet
         self.connect((host, port))
-    
+
     def handle_connect(self):
         self.register()
 
@@ -189,19 +157,22 @@ class Bridge(Telnet):
         socket_error = None
 
         exec_string += '\r\n'
-        self.send(exec_string)
+        try:
+            self.send(exec_string)
+        except Exception as e:
+            print str(e)
+            print "String: %s" % exec_string
 
         while _uuid not in self.callbacks.keys():
-
             Bridge.timeout_ctr += interval
             if Bridge.timeout_ctr > self.timeout:
                 print 'Timeout: %s' % exec_string
                 raise JSBridgeDisconnectError("Connection timed out")
-            
+
             sleep(interval)
             try:
                 self.send('')
-            except socket.error, e:
+            except socket.error:
                 # Necessary for Python <2.7.2. See bug 764643
                 socket_error = True
 
@@ -210,38 +181,47 @@ class Bridge(Telnet):
 
         # reset the counter
         Bridge.timeout_ctr = 0.
-        
+
         callback = self.callbacks.pop(_uuid)
         if callback['result'] is False and raise_exeption is True:
             raise JavaScriptException(callback['exception'])
-        return callback 
-        
+        return callback
+
     def register(self):
         _uuid = str(uuid.uuid1())
-        self.send('bridge.register("'+_uuid+'", "'+self.bridge_type+'")\r\n')
+        self.send('bridge.register("' + _uuid + '", "' +
+                  self.bridge_type + '")\r\n')
         self.registered = True
 
     def execFunction(self, func_name, args, interval=.25):
         _uuid = str(uuid.uuid1())
         exec_args = [encoder.encode(_uuid), func_name, encoder.encode(args)]
-        return self.run(_uuid, 'bridge.execFunction('+ ', '.join(exec_args)+')', interval)
-        
+        return self.run(_uuid, 'bridge.execFunction(' +
+                        ', '.join(exec_args) + ')', interval)
+
     def setAttribute(self, obj_name, name, value):
         _uuid = str(uuid.uuid1())
-        exec_args = [encoder.encode(_uuid), obj_name, encoder.encode(name), encoder.encode(value)]
-        return self.run(_uuid, 'bridge.setAttribute('+', '.join(exec_args)+')')
-        
+        exec_args = [encoder.encode(_uuid), obj_name,
+                     encoder.encode(name), encoder.encode(value)]
+        return self.run(_uuid, 'bridge.setAttribute('
+                        + ', '.join(exec_args) + ')')
+
     def set(self, obj_name):
         _uuid = str(uuid.uuid1())
-        return self.run(_uuid, 'bridge.set('+', '.join([encoder.encode(_uuid), obj_name])+')')
-        
+        return self.run(_uuid, 'bridge.set(' +
+                        ', '.join([encoder.encode(_uuid), obj_name]) + ')')
+
     def describe(self, obj_name):
         _uuid = str(uuid.uuid1())
-        return self.run(_uuid, 'bridge.describe('+', '.join([encoder.encode(_uuid), obj_name])+')')
-    
+        return self.run(_uuid, 'bridge.describe(' +
+                        ', '.join([encoder.encode(_uuid), obj_name]) + ')')
+
     def fire_callbacks(self, obj):
+        if 'uuid' not in obj and 'exception' in obj:
+            # harness failure
+            raise JavaScriptException(obj['exception']['message'])
         self.callbacks[obj['uuid']] = obj
-    
+
     def process_read(self, data):
         """Parse out json objects and fire callbacks."""
         self.sbuffer += data
@@ -252,64 +232,78 @@ class Bridge(Telnet):
             index = self.sbuffer.find('{')
             if index is not -1 and index is not 0:
                 self.sbuffer = self.sbuffer[index:]
-            # Try to get a json object from the data stream    
+                # Try to get a json object from the data stream
             try:
                 obj, index = decoder.raw_decode(self.sbuffer)
-            except Exception, e:
+            except Exception:
                 self.parsing = False
-            # If we got an object fire the callback infra    
+                # If we got an object fire the callback infra
             if self.parsing:
                 self.fire_callbacks(obj)
                 self.sbuffer = self.sbuffer[index:]
-        
+
+
 class BackChannel(Bridge):
-    
     bridge_type = "backchannel"
-    
+
     def __init__(self, host, port):
         Bridge.__init__(self, host, port)
         self.uuid_listener_index = {}
         self.event_listener_index = {}
         self.global_listeners = []
-        
+
     def fire_callbacks(self, obj):
-        """Handle all callback fireing on json objects pulled from the data stream."""
-        self.fire_event(**dict([(str(key), value,) for key, value in obj.items()]))
+        """Handle all callback firing on json objects pulled
+        from the data stream.
+
+        """
+        # no 'self' (key: 0)
+        args = inspect.getargspec(self.fire_event).args[1:]
+
+        self.fire_event(**dict([(str(key), value,)
+                            for key, value in obj.items()
+                                if key in args]))
 
     def add_listener(self, callback, uuid=None, eventType=None):
         if uuid is not None:
             self.uuid_listener_index.setdefault(uuid, []).append(callback)
         if eventType is not None:
-            self.event_listener_index.setdefault(eventType, []).append(callback)
+            self.event_listener_index.setdefault(eventType,
+                                                 []).append(callback)
 
     def add_global_listener(self, callback):
         self.global_listeners.append(callback)
 
-    def fire_event(self, eventType=None, uuid=None, result=None, exception=None):
-        Bridge.timeout_ctr = 0. # reset the counter
-        event = eventType
-        if uuid is not None and self.uuid_listener_index.has_key(uuid):
+    def fire_event(self, eventType=None, uuid=None, result=None,
+                   exception=None):
+        # reset the counter
+        Bridge.timeout_ctr = 0.
+
+        if uuid is not None and uuid in self.uuid_listener_index:
             for callback in self.uuid_listener_index[uuid]:
                 callback(result)
-        if event is not None and self.event_listener_index.has_key(event):
-            for callback in self.event_listener_index[event]:
+        if eventType is not None and eventType in self.event_listener_index:
+            for callback in self.event_listener_index[eventType]:
                 callback(result)
         for listener in self.global_listeners:
             listener(eventType, result)
 
 thread = None
- 
+
+
 def create_network(hostname, port):
     back_channel = BackChannel(hostname, port)
     bridge = Bridge(hostname, port)
     global thread
     if not thread or not thread.isAlive():
         def do():
-            try: asyncore.loop(use_poll=True)
-            except select.error:pass
-            
+            try:
+                asyncore.loop(use_poll=True)
+            except select.error:
+                pass
+
         thread = Thread(target=do)
-        getattr(thread, 'setDaemon', lambda x : None)(True)
+        getattr(thread, 'setDaemon', lambda x: None)(True)
         thread.start()
-    
+
     return back_channel, bridge
